USE master;
SET ANSI_NULLS ON;
SET QUOTED_IDENTIFIER ON;
GO
--
--
-- Impression LCF SQL Server Upgrade Script
-- Copyright 2013-2016 Logicdriven, LLC.  All rights reserved.
-- Last modified: 5 June 2016
--
--
-- PLEASE READ COMPLETLY BEFORE EXECUTING THIS SCRIPT
--
--
-- This script updates the stored procedures of an Impression connector database.  The script
-- performs the following actions:
--
--     1: Modifies the proc_NewImpressionProjectDatabase and proc_NewImpressionDBFromConsole
--        stored procedures to include creation of the Impression Talk tables.
--
--     2: Modifies or creates proc_SetImpressionDBAccessRights, which can be used to set 
--        permissions on an Impression content database for a specific login.
--
--     3: REMOVES proc_CopyImpressionDB.  We now recommend using the database copying tool 
--        to copy Impression content databases.
--
--
-- CUSTOMIZATION:
--
-- The name of the connector database can be customized to match your existing connector
-- name.  To do this, change the SET statement for the variable @ConnectorDBName immediately
-- following these comments.
--
--
-- THIS IS AN UPGRADE SCRIPT:
--
-- This script is for upgrading existing connector databases.  If you are creating a new connector
-- database, use the "Impression SQL Server Install Script" instead of this one.
--
--
--


--
-- declare local variables (customize these values as desired)
--
DECLARE @ConnectorDBName nvarchar(128);
SET @ConnectorDBName = 'impressionConnector';


PRINT 'Impression SQL Server Upgrade Script';
PRINT 'Execution started';
PRINT ' ';






DECLARE @cSQL nvarchar(MAX);

--
-- create/upgrade/remove stored procedures
--
-- to create these procedures within a connector database defined as a variable
-- (@ConnectorDBName), we have to do a bit of slight-of-hand.  we use @cSQL to 
-- define the script, and then use another string variable, @execSQL, to execute 
-- the cSQL script using the newly created connector.  the @execSQL text is then 
-- executed here from the master catalog.
--
-- our @execSQL text will be the same, namely:
--
--     SET @execSQL = 'USE [' + @ConnectorDBName + ']; EXECUTE (''' + REPLACE(@cSQL, '''', '''''') + ''')';
--
-- where the REPLACE code ensure that the quotes in the script are properly escaped
--
-- 
DECLARE @execSQL nvarchar(MAX);
DECLARE @crlf nvarchar(2);
SET @crlf = CHAR(13) + CHAR(10); 




--
-- remove proc_CopyImpressionDB stored procedure
--
-- this does not use the @execSQL method shown above; we're simply removing
-- the existing stored procedure.
--
--
PRINT 'removing proc_CopyImpressionDB';
SET @cSQL = CAST('' AS nvarchar(max))
+ 'USE [' + @ConnectorDBName + '];'
+ 'IF EXISTS (SELECT * FROM [' + @ConnectorDBName + '].[sys].[procedures] WHERE Name = ''proc_CopyImpressionDB'')' + @crlf
+ '  BEGIN ' + @crlf
+ '    DROP PROCEDURE [dbo].[proc_CopyImpressionDB];' + @crlf 
+ '    PRINT ''proc_CopyImpressionDB removed'';' + @crlf
+ '  END' + @crlf
+ 'ELSE' + @crlf
+ '  PRINT ''proc_CopyImpressionDB not found (remove not needed)'';' + @crlf
EXECUTE(@cSQL);


--
-- upgrade proc_NewImpressionProjectDatabase stored procedure
--
--
PRINT 'upgrading proc_NewImpressionProjectDatabase';

SET @cSQL = CAST('' AS nvarchar(max))
+ 'ALTER PROCEDURE [dbo].[proc_NewImpressionProjectDatabase](@ProjectName nvarchar(50), @SQLLoginName nvarchar(50), @SQLLoginNamePassword as nvarchar(50)) AS' + @crlf
+ '-- proc_NewImpressionProjectDatabase' + @crlf
+ '--' + @crlf
+ '-- Impression Learning Content Framework' + @crlf
+ '-- Copyright 2006-2016 Logicdriven, LLC.  All rights reserved.' + @crlf
+ '--' + @crlf
+ '-- This procedure creates a new Impression-format content database.' + @crlf
+ '--' + @crlf
+ '-- PARAMETERS:' + @crlf
+ '--     @ProjectName          - catalog name of the new content database' + @crlf
+ '--     @SQLLoginName         - login to use to access the database.  if the login does' + @crlf
+ '--                             not exist, it will be created' + @crlf
+ '--     @SQLLoginNamePassword - password for @SQLLoginName. ignored if @SQLLoginName exists.' + @crlf
+ '--' + @crlf
+ '-- NOTES:' + @crlf
+ '--     - This procedure must be run with Administrator rights.' + @crlf
+ '--     - The ''sa'' account is incompatible with certain system procs.  ' + @crlf
+ '--       DO NOT use the ''sa'' account as the @SQLLoginName parameter.' + @crlf
+ '--' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ '--' + @crlf
+ '-- initial prep. validate parameters, then create the database' + @crlf
+ '--' + @crlf
+ '--' + @crlf
+ 'SET NOCOUNT ON' + @crlf
+ 'DECLARE @cSQL nvarchar(MAX);' + @crlf
+ @crlf
+ @crlf
+ 'IF (LEN(LTRIM(@ProjectName)) = 0) BEGIN' + @crlf
+ '	RAISERROR (''Catalog name cannot be blank.'', 16, 1);' + @crlf
+ '	RETURN;' + @crlf
+ 'END' + @crlf
+ 'IF (LEN(LTRIM(@SQLLoginName)) = 0) BEGIN' + @crlf
+ '	RAISERROR (''SQL login name cannot be blank.'', 16, 1);' + @crlf
+ '	RETURN;' + @crlf
+ 'END' + @crlf
+ @crlf
+ 'IF (LEN(LTRIM(@SQLLoginNamePassword)) = 0) BEGIN' + @crlf
+ '	RAISERROR (''SQL login password cannot be blank.'', 16, 1);' + @crlf
+ '	RETURN;' + @crlf
+ 'END' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ 'SET @cSQL = ''CREATE DATABASE ['' + @ProjectName + '']'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ '--' + @crlf
+ '-- create content database structure' + @crlf
+ '--' + @crlf
+ '-- after creating the database, we now go through table by table, adding' + @crlf
+ '-- the columns, keys, and indexes for each one' + @crlf
+ '--' + @crlf
+ '--' + @crlf
+ @crlf
+ @crlf
+ '-- impressionAutonumbers' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @ProjectName + ''].[dbo].[impressionAutonumbers] (''' + @crlf
+ '	+ ''[nextNumber] [int] IDENTITY (1, 1) NOT NULL,''' + @crlf
+ '	+ ''[ignored] [int] NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY]'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- impressionLessons' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @ProjectName + ''].[dbo].[impressionLessons] (''' + @crlf
+ '	+ ''[lessonID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[lessonTitle] [nvarchar] (255) NOT NULL,''' + @crlf
+ '	+ ''[lessonIdentifier] [nvarchar] (255) NOT NULL,''' + @crlf
+ '	+ ''[parentID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[indentLevel] [int] NOT NULL,''' + @crlf
+ '	+ ''[sortPosition] [int] NOT NULL,''' + @crlf
+ '	+ ''[lessonType] [int] NOT NULL,''' + @crlf
+ '	+ ''[lessonProperties] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[lessonData] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[guideData] [nvarchar](max) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'SET @cSQL = ''ALTER TABLE ['' + @ProjectName + ''].[dbo].[impressionLessons] WITH NOCHECK ADD CONSTRAINT [PK_impressionLessons] PRIMARY KEY CLUSTERED ([lessonID]) ON [PRIMARY];'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- impressionMilestones' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @ProjectName + ''].[dbo].[impressionMilestones] (''' + @crlf
+ '	+ ''[milestoneID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[lessonID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[savedBy] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[savedDate] [datetime] NOT NULL,''' + @crlf
+ '	+ ''[milestoneProperties] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[lessonData] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[guideData] [nvarchar](max) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'SET @cSQL = ''ALTER TABLE ['' + @ProjectName + ''].[dbo].[impressionMilestones] ADD CONSTRAINT [PK_impressionMilestones] PRIMARY KEY NONCLUSTERED ([milestoneID]) ON [PRIMARY];''' + @crlf
+ '	+ ''CREATE CLUSTERED INDEX [IX_lessonID-milestoneID] ON ['' + @ProjectName + ''].[dbo].[impressionMilestones]([lessonID], [milestoneID]) WITH FILLFACTOR = 80 ON [PRIMARY];''' + @crlf
+ '	+ ''CREATE INDEX [IX_milestoneID-lessonID] ON ['' + @ProjectName + ''].[dbo].[impressionMilestones]([milestoneID], [lessonID]) WITH FILLFACTOR = 80 ON [PRIMARY];'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- impressionProjectProperties' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @ProjectName + ''].[dbo].[impressionProjectProperties] (''' + @crlf
+ '	+ ''[propertyData] [nvarchar](max) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- impressionUserLocks' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @ProjectName + ''].[dbo].[impressionUserLocks] (''' + @crlf
+ '	+ ''[lessonID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[username] [nvarchar] (32) NOT NULL,''' + @crlf
+ '	+ ''[machineName] [nvarchar] (32) NOT NULL,''' + @crlf
+ '	+ ''[machineIP] [nvarchar] (16) NOT NULL,''' + @crlf
+ '	+ ''[sessionID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[lockDate] [datetime] NOT NULL,''' + @crlf
+ '	+ ''[lockType] [int] NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY]'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- talkConfiguration' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @ProjectName + ''].[dbo].[talkConfiguration] (''' + @crlf
+ '	+ ''[configurationID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[configurationData] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[lastModified] [datetime] NOT NULL,''' + @crlf
+ '	+ ''[revision] [nvarchar](50) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'SET @cSQL = ''ALTER TABLE ['' + @ProjectName + ''].[dbo].[talkConfiguration] WITH NOCHECK ADD CONSTRAINT [PK_talkConfiguration] PRIMARY KEY CLUSTERED ([configurationID]) ON [PRIMARY];'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- talkPosts' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @ProjectName + ''].[dbo].[talkPosts] (''' + @crlf
+ '	+ ''[postID] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[applicationID] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[lessonID] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[sbid] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[postData] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[status] [int] NOT NULL,''' + @crlf
+ '	+ ''[lastModified] [datetime] NOT NULL,''' + @crlf
+ '	+ ''[revision] [nvarchar](50) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'SET @cSQL = ''ALTER TABLE ['' + @ProjectName + ''].[dbo].[talkPosts] WITH NOCHECK ADD CONSTRAINT [PK_talkPosts] PRIMARY KEY NONCLUSTERED ([postID]) ON [PRIMARY];'' ' + @crlf
+ '	+ ''CREATE CLUSTERED INDEX [IX_applicationID-lessonID] ON ['' + @ProjectName + ''].[dbo].[talkPosts]([applicationID], [lessonID]) WITH FILLFACTOR = 80 ON [PRIMARY];'' ' + @crlf
+ '	+ ''CREATE INDEX [IX_applicationID] ON ['' + @ProjectName + ''].[dbo].[talkPosts]([applicationID]) WITH FILLFACTOR = 80 ON [PRIMARY];'' ' + @crlf
+ '	+ ''CREATE INDEX [IX_lessonID] ON ['' + @ProjectName + ''].[dbo].[talkPosts]([lessonID]) WITH FILLFACTOR = 80 ON [PRIMARY];'' ' + @crlf
+ '	+ ''CREATE INDEX [IX_applicationID-lessonID-sbid] ON ['' + @ProjectName + ''].[dbo].[talkPosts]([applicationID], [lessonID], [sbid]) WITH FILLFACTOR = 80 ON [PRIMARY];'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- talkComments' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @ProjectName + ''].[dbo].[talkComments] (''' + @crlf
+ '	+ ''[commentID] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[postID] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[commentData] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[lastModified] [datetime] NOT NULL,''' + @crlf
+ '	+ ''[revision] [nvarchar](50) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'SET @cSQL = ''ALTER TABLE ['' + @ProjectName + ''].[dbo].[talkComments] WITH NOCHECK ADD CONSTRAINT [PK_talkComments] PRIMARY KEY NONCLUSTERED ([commentID]) ON [PRIMARY];'' ' + @crlf
+ '	+ ''CREATE CLUSTERED INDEX [IX_postID] ON ['' + @ProjectName + ''].[dbo].[talkComments]([postID]) WITH FILLFACTOR = 80 ON [PRIMARY]'';' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ '--' + @crlf
+ '-- content database login' + @crlf
+ '--' + @crlf
+ '-- we''ll create the login if it doesn''t exist, and then go through the' + @crlf
+ '-- tables adding the appropriate rights' + @crlf
+ '--' + @crlf
+ '-- note that we allow ALTER permissions for the impressionAutonumbers' + @crlf
+ '-- table.  this is done so that we can reset the identity seed value; ' + @crlf
+ '-- this may be required if data is transferred from an existing content' + @crlf
+ '-- database.' + @crlf
+ '--' + @crlf
+ '--' + @crlf
+ @crlf
+ @crlf
+ '-- create the login as needed, then set permissions' + @crlf
+ '--' + @crlf
+ 'IF NOT EXISTS(SELECT name FROM master.sys.server_principals WHERE name = @SQLLoginName) BEGIN' + @crlf
+ '	SET @cSQL = ''CREATE LOGIN '' + @SQLLoginName + '' WITH PASSWORD = '''''' + @SQLLoginNamePassword + '''''', DEFAULT_DATABASE = ['' + @ProjectName + ''];'';' + @crlf
+ '	EXECUTE(@cSQL);' + @crlf
+ 'END' + @crlf
+ @crlf
+ 'SET @cSQL = ''USE '' + @ProjectName + '';''' + @crlf
+ '	+ '' EXECUTE sp_grantdbaccess N'''''' + @SQLLoginName + '''''', N'''''' + @SQLLoginName + '''''';''' + @crlf
+ '	+ '' GRANT ALTER, SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionAutonumbers] TO '' + @SQLLoginName + '';''' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionLessons] TO '' + @SQLLoginName + '';''' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionMilestones] TO '' + @SQLLoginName + '';''' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionProjectProperties] TO '' + @SQLLoginName + '';''' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionUserLocks] TO '' + @SQLLoginName + '';''' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[talkConfiguration]  TO '' + @SQLLoginName + '';''' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[talkPosts] TO '' + @SQLLoginName + '';''' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[talkComments] TO '' + @SQLLoginName + '';''' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- processing complete' + @crlf

SET @execSQL = 'USE [' + @ConnectorDBName + ']; EXECUTE (''' + REPLACE(@cSQL, '''', '''''') + ''')';
EXECUTE(@execSQL);


--
-- upgrade proc_NewImpressionDBFromConsole stored procedure
--
--
PRINT 'upgrading proc_NewImpressionDBFromConsole';

SET @cSQL = CAST('' AS nvarchar(max))
+ 'ALTER PROCEDURE [dbo].[proc_NewImpressionDBFromConsole](@DatabaseName nvarchar(50), @LoginName nvarchar(50), @LoginPassword as nvarchar(50)) AS' + @crlf
+ '-- proc_NewImpressionDBFromConsole' + @crlf
+ '--' + @crlf
+ '-- Impression Learning Content Framework' + @crlf
+ '-- Copyright 2006-2016 Logicdriven, LLC.  All rights reserved.' + @crlf
+ '--' + @crlf
+ '-- This procedure creates a new Impression-format content database.  It is intended' + @crlf
+ '-- to be run from SQL Server Management Studio.  The procedure only creates the database' + @crlf
+ '-- (and the login, if needed) - it does not add references to the database into the' + @crlf
+ '-- connector.' + @crlf
+ '--' + @crlf
+ '-- PARAMETERS:' + @crlf
+ '--     @DatabaseName  - name of the new database' + @crlf
+ '--     @LoginName     - name of the login to use to access the database.  if the user' + @crlf
+ '--                      does not exist, it will be created' + @crlf
+ '--     @LoginPassword - password for @LoginName' + @crlf
+ '--' + @crlf
+ '-- NOTES:' + @crlf
+ '--     - This procedure must be run with Administrator rights.' + @crlf
+ '--     - The ''sa'' account is incompatible with certain system procs.  ' + @crlf
+ '--       DO NOT use the ''sa'' account as the @LoginName parameter.' + @crlf
+ '--' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ 'PRINT ''Execution started'';' + @crlf
+ 'PRINT '' '';' + @crlf
+ @crlf
+ @crlf
+ '--' + @crlf
+ '-- initial prep. validate parameters, then create the database' + @crlf
+ '--' + @crlf
+ '--' + @crlf
+ 'SET NOCOUNT ON' + @crlf
+ 'DECLARE @cSQL nvarchar(max);' + @crlf
+ 'DECLARE @crlf nvarchar(2);' + @crlf
+ 'SET @crlf = CHAR(13) + CHAR(10);' + @crlf
+ @crlf
+ 'IF (LEN(LTRIM(@DatabaseName)) = 0) BEGIN' + @crlf
+ '	RAISERROR (''Database name cannot be blank.'', 16, 1);' + @crlf
+ '	RETURN;' + @crlf
+ 'END' + @crlf
+ 'IF (LEN(LTRIM(@LoginName)) = 0) BEGIN' + @crlf
+ '	RAISERROR (''Login name cannot be blank.'', 16, 1);' + @crlf
+ '	RETURN;' + @crlf
+ 'END' + @crlf
+ @crlf
+ 'IF (LEN(LTRIM(@LoginPassword)) = 0) BEGIN' + @crlf
+ '	RAISERROR (''Login password cannot be blank.'', 16, 1);' + @crlf
+ '	RETURN;' + @crlf
+ 'END' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ 'SET @cSQL = ''CREATE DATABASE ['' + @DatabaseName + '']'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ '--' + @crlf
+ '-- create content database structure' + @crlf
+ '--' + @crlf
+ '-- after creating the database, we now go through table by table, adding' + @crlf
+ '-- the columns, keys, and indexes for each one' + @crlf
+ '--' + @crlf
+ '--' + @crlf
+ @crlf
+ @crlf
+ '-- impressionAutonumbers' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @DatabaseName + ''].[dbo].[impressionAutonumbers] (''' + @crlf
+ '	+ ''[nextNumber] [int] IDENTITY (1, 1) NOT NULL,''' + @crlf
+ '	+ ''[ignored] [int] NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY]'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- impressionLessons' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @DatabaseName + ''].[dbo].[impressionLessons] (''' + @crlf
+ '	+ ''[lessonID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[lessonTitle] [nvarchar] (255) NOT NULL,''' + @crlf
+ '	+ ''[lessonIdentifier] [nvarchar] (255) NOT NULL,''' + @crlf
+ '	+ ''[parentID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[indentLevel] [int] NOT NULL,''' + @crlf
+ '	+ ''[sortPosition] [int] NOT NULL,''' + @crlf
+ '	+ ''[lessonType] [int] NOT NULL,''' + @crlf
+ '	+ ''[lessonProperties] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[lessonData] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[guideData] [nvarchar](max) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'SET @cSQL = ''ALTER TABLE ['' + @DatabaseName + ''].[dbo].[impressionLessons] WITH NOCHECK ADD CONSTRAINT [PK_impressionLessons] PRIMARY KEY CLUSTERED ([lessonID]) ON [PRIMARY];'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- impressionMilestones' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @DatabaseName + ''].[dbo].[impressionMilestones] (''' + @crlf
+ '	+ ''[milestoneID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[lessonID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[savedBy] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[savedDate] [datetime] NOT NULL,''' + @crlf
+ '	+ ''[milestoneProperties] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[lessonData] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[guideData] [nvarchar](max) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'SET @cSQL = ''ALTER TABLE ['' + @DatabaseName + ''].[dbo].[impressionMilestones] ADD CONSTRAINT [PK_impressionMilestones] PRIMARY KEY NONCLUSTERED ([milestoneID]) ON [PRIMARY];'' + @crlf' + @crlf
+ '	+ ''CREATE CLUSTERED INDEX [IX_lessonID-milestoneID] ON ['' + @DatabaseName + ''].[dbo].[impressionMilestones]([lessonID], [milestoneID]) WITH FILLFACTOR = 80 ON [PRIMARY];'' + @crlf' + @crlf
+ '	+ ''CREATE INDEX [IX_milestoneID-lessonID] ON ['' + @DatabaseName + ''].[dbo].[impressionMilestones]([milestoneID], [lessonID]) WITH FILLFACTOR = 80 ON [PRIMARY];'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- impressionProjectProperties' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @DatabaseName + ''].[dbo].[impressionProjectProperties] (''' + @crlf
+ '	+ ''[propertyData] [nvarchar](max) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- impressionUserLocks' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @DatabaseName + ''].[dbo].[impressionUserLocks] (''' + @crlf
+ '	+ ''[lessonID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[username] [nvarchar] (32) NOT NULL,''' + @crlf
+ '	+ ''[machineName] [nvarchar] (32) NOT NULL,''' + @crlf
+ '	+ ''[machineIP] [nvarchar] (16) NOT NULL,''' + @crlf
+ '	+ ''[sessionID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[lockDate] [datetime] NOT NULL,''' + @crlf
+ '	+ ''[lockType] [int] NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY]'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- talkConfiguration' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @DatabaseName + ''].[dbo].[talkConfiguration] (''' + @crlf
+ '	+ ''[configurationID] [nvarchar] (50) NOT NULL,''' + @crlf
+ '	+ ''[configurationData] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[lastModified] [datetime] NOT NULL,''' + @crlf
+ '	+ ''[revision] [nvarchar](50) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'SET @cSQL = ''ALTER TABLE ['' + @DatabaseName + ''].[dbo].[talkConfiguration] WITH NOCHECK ADD CONSTRAINT [PK_talkConfiguration] PRIMARY KEY CLUSTERED ([configurationID]) ON [PRIMARY];'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- talkPosts' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @DatabaseName + ''].[dbo].[talkPosts] (''' + @crlf
+ '	+ ''[postID] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[applicationID] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[lessonID] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[sbid] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[postData] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[status] [int] NOT NULL,''' + @crlf
+ '	+ ''[lastModified] [datetime] NOT NULL,''' + @crlf
+ '	+ ''[revision] [nvarchar](50) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'SET @cSQL = ''ALTER TABLE ['' + @DatabaseName + ''].[dbo].[talkPosts] WITH NOCHECK ADD CONSTRAINT [PK_talkPosts] PRIMARY KEY NONCLUSTERED ([postID]) ON [PRIMARY];'' + @crlf' + @crlf
+ '	+ ''CREATE CLUSTERED INDEX [IX_applicationID-lessonID] ON ['' + @DatabaseName + ''].[dbo].[talkPosts]([applicationID], [lessonID]) WITH FILLFACTOR = 80 ON [PRIMARY];'' ' + @crlf
+ '	+ ''CREATE INDEX [IX_applicationID] ON ['' + @DatabaseName + ''].[dbo].[talkPosts]([applicationID]) WITH FILLFACTOR = 80 ON [PRIMARY];'' ' + @crlf
+ '	+ ''CREATE INDEX [IX_lessonID] ON ['' + @DatabaseName + ''].[dbo].[talkPosts]([lessonID]) WITH FILLFACTOR = 80 ON [PRIMARY];'' ' + @crlf
+ '	+ ''CREATE INDEX [IX_applicationID-lessonID-sbid] ON ['' + @DatabaseName + ''].[dbo].[talkPosts]([applicationID], [lessonID], [sbid]) WITH FILLFACTOR = 80 ON [PRIMARY];'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ '-- talkComments' + @crlf
+ '--' + @crlf
+ 'SET @cSQL = ''CREATE TABLE ['' + @DatabaseName + ''].[dbo].[talkComments] (''' + @crlf
+ '	+ ''[commentID] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[postID] [nvarchar](50) NOT NULL,''' + @crlf
+ '	+ ''[commentData] [nvarchar](max) NOT NULL,''' + @crlf
+ '	+ ''[lastModified] [datetime] NOT NULL,''' + @crlf
+ '	+ ''[revision] [nvarchar](50) NOT NULL''' + @crlf
+ '	+ '') ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'SET @cSQL = ''ALTER TABLE ['' + @DatabaseName + ''].[dbo].[talkComments] WITH NOCHECK ADD CONSTRAINT [PK_talkComments] PRIMARY KEY NONCLUSTERED ([commentID]) ON [PRIMARY];'' + @crlf' + @crlf
+ '	+ ''CREATE CLUSTERED INDEX [IX_postID] ON ['' + @DatabaseName + ''].[dbo].[talkComments]([postID]) WITH FILLFACTOR = 80 ON [PRIMARY]'';' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ '--' + @crlf
+ '-- content database login' + @crlf
+ '--' + @crlf
+ '-- we''ll create the login if it doesn''t exist, and then go through the' + @crlf
+ '-- tables adding the appropriate rights' + @crlf
+ '--' + @crlf
+ '-- note that we allow ALTER permissions for the impressionAutonumbers' + @crlf
+ '-- table.  this is done so that we can reset the identity seed value; ' + @crlf
+ '-- this may be required if data is transferred from an existing content' + @crlf
+ '-- database.' + @crlf
+ '--' + @crlf
+ '--' + @crlf
+ @crlf
+ @crlf
+ '-- create the login as needed, then set permissions' + @crlf
+ '--' + @crlf
+ 'IF NOT EXISTS(SELECT name FROM master.sys.server_principals WHERE name = @LoginName) BEGIN' + @crlf
+ '	SET @cSQL = ''CREATE LOGIN '' + @LoginName + '' WITH PASSWORD = '''''' + @LoginPassword + '''''', DEFAULT_DATABASE = ['' + @DatabaseName + ''];'';' + @crlf
+ '	PRINT @cSQL;' + @crlf
+ '	EXECUTE(@cSQL);' + @crlf
+ 'END' + @crlf
+ @crlf
+ 'SET @cSQL = ''USE '' + @DatabaseName + '';'' + @crlf' + @crlf
+ '	+ '' EXECUTE sp_grantdbaccess N'''''' + @LoginName + '''''';'' + @crlf' + @crlf
+ '	+ '' GRANT ALTER, SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionAutonumbers] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionLessons] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionMilestones] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionProjectProperties] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionUserLocks] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[talkConfiguration]  TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[talkPosts] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[talkComments] TO '' + @LoginName + '';''' + @crlf
+ 'PRINT @cSQL;' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ '-- processing complete' + @crlf
+ '--' + @crlf
+ 'PRINT '' '';' + @crlf
+ 'PRINT '' '';' + @crlf
+ 'PRINT ''Execution complete.'';' + @crlf
+ 'PRINT '' '';' + @crlf
+ 'PRINT ''The Impression content database '''''' + @DatabaseName + '''''' has been created.'';' + @crlf
+ 'PRINT ''Login '''''' + @LoginName + '''''' has been granted access to the database.'';' + @crlf
+ 'PRINT '' '';' + @crlf
+ 'PRINT ''Please set up a maintenance plan to safeguard your data.'';' + @crlf

SET @execSQL = 'USE [' + @ConnectorDBName + ']; EXECUTE (''' + REPLACE(@cSQL, '''', '''''') + ''')';
EXECUTE(@execSQL);


--
-- create (then update) proc_SetImpressionDBAccessRights stored procedure
--
--

SET @cSQL = CAST('' AS nvarchar(max))
+ 'IF NOT exists(select * from sys.procedures where object_id = OBJECT_ID(''[dbo].[proc_SetImpressionDBAccessRights]'')) BEGIN' + @crlf
+ '	PRINT ''creating proc_SetImpressionDBAccessRights'';' + @crlf
+ '	EXECUTE(''CREATE PROCEDURE [dbo].[proc_SetImpressionDBAccessRights] AS'');' + @crlf
+ 'END;' + @crlf;
SET @execSQL = 'USE [' + @ConnectorDBName + ']; EXECUTE (''' + REPLACE(@cSQL, '''', '''''') + ''')';
EXECUTE(@execSQL);

PRINT 'upgrading proc_SetImpressionDBAccessRights';

SET @cSQL = CAST('' AS nvarchar(max))
+ 'ALTER PROCEDURE [dbo].[proc_SetImpressionDBAccessRights](@DatabaseName nvarchar(50), @LoginName nvarchar(50), @LoginPassword as nvarchar(50) = '''', @CreateLogin bit = 0) AS' + @crlf
+ '-- ' + @crlf
+ '-- proc_SetImpressionDBAccessRights' + @crlf
+ '--' + @crlf
+ '-- Impression Learning Content Framework' + @crlf
+ '-- Copyright 2013-2016 Logicdriven, LLC.  All rights reserved.' + @crlf
+ '--' + @crlf
+ '-- This procedure sets access rights to an existing Impression-format content' + @crlf
+ '-- database for the specified login.' + @crlf
+ '--' + @crlf
+ '-- PARAMETERS:' + @crlf
+ '--     @DatabaseName  - name of the content database to set permissions to.' + @crlf
+ '--     @LoginName     - name of the login to set permissions of.' + @crlf
+ '--     @LoginPassword - password for @LoginName.  This can be omitted if @LoginName' + @crlf
+ '--                      already exists.' + @crlf
+ '--     @CreateLogin   - True (1) if the login should be created if it doesn''t' + @crlf
+ '--                      already exist, or 0 if it should not be created.' + @crlf
+ '--                    ' + @crlf
+ '--' + @crlf
+ '-- NOTES:' + @crlf
+ '--     - This procedure must be run with Administrator rights.' + @crlf
+ '--     - The ''sa'' account is incompatible with certain system procs.  ' + @crlf
+ '--       DO NOT use the ''sa'' account as the @LoginName parameter.' + @crlf
+ '--' + @crlf
+ '--' + @crlf
+ '--' + @crlf
+ @crlf
+ @crlf
+ 'PRINT ''Execution started'';' + @crlf
+ 'PRINT '' '';' + @crlf
+ '--' + @crlf
+ '-- initial prep - declare locals, validate parameters' + @crlf
+ '--' + @crlf
+ 'SET NOCOUNT ON ' + @crlf
+ 'DECLARE @cSQL nvarchar(max);' + @crlf
+ 'DECLARE @crlf nvarchar(2);' + @crlf
+ 'SET @crlf = CHAR(13) + CHAR(10);' + @crlf
+ @crlf
+ @crlf
+ 'PRINT ''Validating parameters'';' + @crlf
+ @crlf
+ 'IF (LEN(LTRIM(@DatabaseName)) = 0) BEGIN' + @crlf
+ '	RAISERROR (''No database name passed.'', 16, 1);' + @crlf
+ '	PRINT ''Execution stopped with error.'';' + @crlf
+ '	RETURN;' + @crlf
+ 'END;' + @crlf
+ @crlf
+ 'IF NOT EXISTS(SELECT name FROM master.sys.databases WHERE name = @DatabaseName) BEGIN' + @crlf
+ '	RAISERROR (''Database not found.'', 16, 1);' + @crlf
+ '	PRINT ''Execution stopped with error.'';' + @crlf
+ '	RETURN;' + @crlf
+ 'END;' + @crlf
+ @crlf
+ 'IF (LEN(LTRIM(@LoginName)) = 0) BEGIN' + @crlf
+ '	RAISERROR (''No login name passed.'', 16, 1);' + @crlf
+ '	PRINT ''Execution stopped with error.'';' + @crlf
+ '	RETURN;' + @crlf
+ 'END;' + @crlf
+ @crlf
+ 'IF NOT EXISTS(SELECT name FROM master.sys.server_principals WHERE name = @LoginName) BEGIN' + @crlf
+ '-- login doesn''t exist.  ensure we have a password and permission to create' + @crlf
+ '	IF (@CreateLogin = 0) BEGIN' + @crlf
+ '		RAISERROR (''Login name not found, and @CreateLogin parameter set to False/0.'', 16, 1);' + @crlf
+ '		PRINT ''Execution stopped with error.'';' + @crlf
+ '		RETURN;' + @crlf
+ '	END;' + @crlf
+ '	IF (LEN(LTRIM(@LoginPassword)) = 0) BEGIN' + @crlf
+ '		RAISERROR (''Password cannot be blank when login does not exist.'', 16, 1);' + @crlf
+ '		PRINT ''Execution stopped with error.'';' + @crlf
+ '		RETURN;' + @crlf
+ '	END;' + @crlf
+ 'END;' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ '--' + @crlf
+ '-- create the login as needed, and set permissions' + @crlf
+ '--' + @crlf
+ 'IF NOT EXISTS(SELECT name FROM master.sys.server_principals WHERE name = @LoginName) BEGIN' + @crlf
+ '	PRINT ''Creating login '''''' + @LoginName + '''''''';' + @crlf
+ '	SET @cSQL = ''CREATE LOGIN '' + @LoginName + '' WITH PASSWORD = '''''' + @LoginPassword + '''''', DEFAULT_DATABASE = ['' + @DatabaseName + ''];'';' + @crlf
+ '	EXECUTE(@cSQL);' + @crlf
+ 'END ELSE BEGIN' + @crlf
+ '	PRINT ''Login '''''' + @LoginName + '''''' already exists.  Skipping creation'';' + @crlf
+ 'END;' + @crlf
+ @crlf
+ @crlf
+ 'PRINT ''Granting database access'';' + @crlf
+ 'SET @cSQL = ''USE '' + @DatabaseName + '';'' + @crlf' + @crlf
+ '	+ '' EXECUTE sp_grantdbaccess N'''''' + @LoginName + '''''';''' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ 'PRINT ''Setting permissions'';' + @crlf
+ 'SET @cSQL = ''USE '' + @DatabaseName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT ALTER, SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionAutonumbers] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionLessons] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionMilestones] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionProjectProperties] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[impressionUserLocks] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[talkConfiguration]  TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[talkPosts] TO '' + @LoginName + '';'' + @crlf' + @crlf
+ '	+ '' GRANT SELECT, UPDATE, INSERT, DELETE ON [dbo].[talkComments] TO '' + @LoginName + '';''' + @crlf
+ 'EXECUTE(@cSQL);' + @crlf
+ @crlf
+ @crlf
+ @crlf
+ @crlf
+ '-- processing complete' + @crlf
+ '--' + @crlf
+ 'PRINT '' '';' + @crlf
+ 'PRINT ''Execution complete.'';' + @crlf
+ 'PRINT ''Login '''''' + @LoginName + '''''' has been granted rights to content database '''''' + @DatabaseName + ''''''.'';' + @crlf
+ @crlf

SET @execSQL = 'USE [' + @ConnectorDBName + ']; EXECUTE (''' + REPLACE(@cSQL, '''', '''''') + ''')';
EXECUTE(@execSQL);








PRINT ' ';
PRINT ' ';
PRINT 'Execution complete.';
PRINT ' ';
PRINT 'The Impression connector database ''' + @ConnectorDBName + ''' has been upgraded.';
PRINT ' ';


